/*
 * clockrec.c
 */

#include "apprec.h"
#include "clockrec.h"
#include "x.h"
#include "menu.h"
#include "window.h"
#include "speech.h"
#include "util.h"
#include "str.h"
#include "mytalk.h"


DefWindowRec clockRec = {

	128 , NULL ,

	ClockCr ,
	ClockDe ,
	ClockUp ,
	ClockMD ,
	ClockMU ,
	ClockKD ,
	ClockAK ,
	ClockAc ,
	ClockSw ,
	ClockId ,
	ClockPr ,
	ClockCo ,
	ClockAE

} ;


typedef struct clockInfo {
	Point			position ;
	short			secsWid ;
	short			minsWid ;
	short			fontNum ;
	short			fontSize ;
	short			baseLine ;
	short			lineFeed ;
	Boolean			showSecs ;
	Boolean			talkHours ;
	Boolean			randomTalk ;
	Boolean			talkOption ;
	Str31			lastTime ;
	unsigned long	lastSecs ;
	FSSpec			prefs ;
	Str31			fontName ;
	Str15			sizeName ;
	Str63			voiceName ;
} ClockInfo ;


static void
WindowSize ( ClockInfo * ci , WindowPtr wp ) {
	SetPort ( wp ) ;
	SizeWindow ( wp , ci -> showSecs ? ci -> secsWid : ci -> minsWid ,
		ci -> lineFeed , 1 ) ;
	InvalRect ( & ( wp -> portRect ) ) ;
}


static void
FillFont ( ClockInfo * ci , WindowPtr wp ) {

short digWid = 0 ;
short ix ;
FontInfo fi ;

	SetPort ( wp ) ;
	TextFont ( ci -> fontNum ) ;
	TextSize ( ci -> fontSize ) ;
	TextMode ( srcOr ) ;
	for ( ix = 0 ; ix < 10 ; ix ++ ) {
		if ( CharWidth ( '0' + ix ) > digWid ) {
			digWid = CharWidth ( '0' + ix ) ;
		}
	}
	GetFontInfo ( & fi ) ;
	ci -> baseLine = fi . ascent + fi . leading + 1 ;
	ci -> lineFeed = ci -> baseLine + fi . descent + 2 ;
	ci -> secsWid = digWid * 7 + 2 * fi . widMax ;
	ci -> minsWid = digWid * 5 + fi . widMax ;
	WindowSize ( ci , wp ) ;
}


/*	A window is being created or opened. Allocate data and		*/
/*	select the window											*/
OSErr
ClockCr ( WindowPtr wp , Handle * data , FSSpec * file ) {

ClockInfo ci ;
FontInfo fi ;
short digWid = 0 ;
short ix ;
long fm ;
Str255 tempName ;

	fm = FreeMem ( ) ;
	if ( fm < 8192L ) { /* Will most likely fail if low on memory */
		return -108 ;
	}

/* Fill in reasonable defaults */
	ci . position . h = ci . position . v = 0 ;
	ci . fontNum = GetAppFont ( ) ;
	ci . fontSize = GetDefFontSize ( ) ;
	GetFontName ( ci . fontNum , ci . fontName ) ;
	NumToString ( ci . fontSize , ci . sizeName ) ;
	ci . showSecs = 0 ;
	ci . lastSecs = 1000 ;
	ci . prefs . vRefNum = 0 ;
	ci . talkHours = SpeechAvailable ( ) ;
	ci . randomTalk = ci . talkHours ;
	ci . talkOption = ci . randomTalk ;
	if ( SpeechAvailable ( ) ) {
		GetCurrentVoiceName ( ci . voiceName ) ;
	}
	if ( file ) {			/* This program always passes the preferences file */
	short refNum = 0 ;
		FSpCreate ( file , 'pref' , 'pref' , smRoman ) ;
		if ( ! FSpOpenDF ( file , fsRdPerm , & refNum ) ) {
		long size = sizeof ( ClockInfo ) ;
			if ( ! FSRead ( refNum , & size , ( Ptr ) & ci ) ) {
			long l ;
			short sh ;
				GetFNum ( ci . fontName , & sh ) ;	/* Get the number of the stored font name */
				GetFontName ( sh , tempName ) ;		/* Go back again */
				if ( EqualString ( tempName , ci . fontName , 0 , 1 ) ) {
					ci . fontNum = sh ;				/* Checked out - use the font */
				} else {
					GetFontName ( ci . fontNum , ci . fontName ) ;
													/* The font seems to have disappeared - use default */
				}
				StringToNum ( ci . sizeName , & l ) ;
				ci . fontSize = l ;
				if ( SpeechAvailable ( ) ) {
					SetCurrentVoiceName ( ci . voiceName ) ;
				}
			}
		}
		if ( refNum ) {
			FSClose ( refNum ) ;
			refNum = 0 ;
		}
		ci . prefs = * file ;
	}

/* Calculate window size and size window */
	FillFont ( & ci , wp ) ;

/* Make window with data */
	PtrToHand ( ( Ptr ) & ci , data , sizeof ( ClockInfo ) ) ;
	if ( ci . position . h || ci . position . v ) {
	Point p = ci . position ;
		p . h += 2 ;
		p . v += 1 ;
		if ( PtInRgn ( ci . position , GetGrayRgn ( ) ) ) {
			p . h += 10 ;
			p . v += 5 ;
			if ( PtInRgn ( p , GetGrayRgn ( ) ) ) {
						/* If part of the window is visible, move to the saved position */
						/* We don't check for a title bar since our window has none */
				MoveWindow ( wp , ci . position . h , ci . position . v , 0 ) ;
			}
		}
	}
	SelectWindow ( wp ) ;
	ShowWindow ( wp ) ;
	return noErr ;
}


/*	Window is being destroyed. Put up a warning dialog, and		*/
/*	return errCancel if the user cancels closing - else call	*/
/*	DisposeWindow here											*/
OSErr
ClockDe ( WindowPtr wp , Handle data ) {

ClockInfo * ci = ( ClockInfo * ) * data ;

	HLock ( data ) ;

/* If we had a file associated at start */

	if ( ci -> prefs . vRefNum ) {
	short refNum = 0 ;

	/* Save the location */
		SetPort ( wp ) ;
		ci -> position . h = ci -> position . v = 0 ;
		LocalToGlobal ( & ( ci -> position ) ) ;

	/* Save the state of the clock */
		if ( ! FSpOpenDF ( & ( ci -> prefs ) , fsWrPerm , & refNum ) ) {
			if ( ! SetEOF ( refNum , sizeof ( ClockInfo ) ) ) {
				if ( ! SetFPos ( refNum , fsFromStart , 0L ) ) {
				long size = sizeof ( ClockInfo ) ;
					FSWrite ( refNum , & size , ( Ptr ) ci ) ;
				}
			}
		}
		if ( refNum ) {
			FSClose ( refNum ) ;
			refNum = 0 ;
			FlushVol ( NULL , ci -> prefs . vRefNum ) ;
		}
	}

/* Dispose all data and window */
	DisposeWindow ( wp ) ;
	DisposeHandle ( data ) ;
	return noErr ;
}


/*	Update the window - BeginUpdate is already called			*/
OSErr
ClockUp ( WindowPtr wp , Handle data , EventRecord * event ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
char state = HGetState ( data ) ;

	HLock ( data ) ;
	TextFont ( ci -> fontNum ) ;
	TextSize ( ci -> fontSize ) ;
	EraseRect ( & ( wp -> portRect ) ) ;
	MoveTo ( ( wp -> portRect . right - StringWidth ( ci -> lastTime ) ) / 2 ,
		ci -> baseLine ) ;
	DrawString ( ci -> lastTime ) ;
	HSetState ( data , state ) ;
	return noErr ;
}


/*	The user clicked in your window. The window port is set and	*/
/*	event -> where is translated into local coordinates now		*/
OSErr
ClockMD ( WindowPtr wp , Handle data , EventRecord * event ) {
	LocalToGlobal ( & ( event -> where ) ) ;
	DragWindow ( wp , event -> where , & gDragLimit ) ;
	return noErr ;
}


/*	MouseUp in a window - you probably should do nothing		*/
OSErr
ClockMU ( WindowPtr wp , Handle data , EventRecord * event ) {
	return noErr ;
}


/*	The user typed in the window - command keys are already		*/
/*	taken care of and don't come here							*/
OSErr
ClockKD ( WindowPtr wp , Handle data , EventRecord * event ) {
	return noErr ;
}


/*	You probably just want to call the KeyDown handler here		*/
OSErr
ClockAK ( WindowPtr wp , Handle data , EventRecord * event ) {
	return noErr ;
}


/*	Activate event - highlight/dehighlight controls & caret		*/
OSErr
ClockAc ( WindowPtr wp , Handle data , EventRecord * event ) {
	return noErr ;
}


/*	You should probably build a suitable activate event and		*/
/*	send to the activate handling for MultiFinder switches		*/
OSErr
ClockSw ( WindowPtr wp , Handle data , EventRecord * event ) {
	return noErr ;
}


/*	Idle time - set the sleep parameter if it's too large for	*/
/*	your needs. GetCaretTime() is a good value for text editors	*/
OSErr
ClockId ( WindowPtr wp , Handle data , long * sleep ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
unsigned long now ;
long sleepTime = ci -> showSecs ? 10 : 200 ;
Str31 time ;
char state = HGetState ( data ) ;

	HLock ( data ) ;

	if ( sleepTime < * sleep ) {
		* sleep = sleepTime ;
	}
	GetDateTime ( & now ) ;
	IUTimeString ( now , ci -> showSecs , time ) ;
	if ( SpeechAvailable ( ) ) {
	DateTimeRec dtr , dtr2 ;
		Secs2Date ( now , & dtr ) ;
		Secs2Date ( ci -> lastSecs , & dtr2 ) ;
		if ( ( ! dtr . minute && dtr2 . minute ) ||
			( ( dtr . minute != dtr2 . minute ) &&
			( ci -> randomTalk && ! ( Random ( ) & 0xf ) ) ) ||
			( ci -> talkOption && KeyIsDown ( 0x3a ) ) ) {
		Str255 s ;
		Str15 num ;
		Str63 oClock ;
			StrGet ( 132 , s , 255 ) ;
			StrGet ( 133 , oClock , 63 ) ;
			NumToString ( dtr . hour % 12 , num ) ;
			if ( ! dtr . minute ) {
				if ( dtr . hour % 12 ) {
					ConcatPStrings ( s , num ) ;
					ConcatPStrings ( s , oClock ) ;
				} else {
					StrGet ( 1000 + dtr . hour , oClock , 63 ) ;
					ConcatPStrings ( s , oClock ) ;
				}
			} else {
				ConcatPStrings ( s , num ) ;
				StrGet ( 134 , oClock , 63 ) ;
				ConcatPStrings ( s , oClock ) ;
				if ( dtr . minute < 10 ) {
					StrGet ( 137 , oClock , 63 ) ;
					ConcatPStrings ( s , oClock ) ;
				}
				NumToString ( dtr . minute , num ) ;
				ConcatPStrings ( s , num ) ;
				StrGet ( dtr . hour > 11 ? 136 : 135 , oClock , 63 ) ;
				ConcatPStrings ( s , oClock ) ;
			}
			if ( ! SpeechBusy ( ) ) {
				FailErr ( MySpeakString ( s ) ) ;
			}
		}
	}
	ci -> lastSecs = now ;
	if ( IUEqualString ( time , ci -> lastTime ) ) {
		CopyPString ( time , ci -> lastTime ) ;
		InvalRect ( & ( wp -> portRect ) ) ;
	}
	HSetState ( data , state ) ;
	return noErr ;
}


/*	This is called when menus need to be updated.				*/
OSErr
ClockPr ( WindowPtr wp , Handle data ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
char state = HGetState ( data ) ;

	HLock ( data ) ;
	TRY {
		FailErr ( AppPr ( wp , app . data ) ) ;
		EnableCmd ( FILE_MENU , CLOSE_ITEM ) ;
		EnableCmd ( EDIT_MENU , SHOW_SECONDS_ITEM ) ;
		CheckCmd ( EDIT_MENU , SHOW_SECONDS_ITEM , ci -> showSecs ) ;
		if ( SpeechAvailable ( ) ) {
			CheckCmd ( EDIT_MENU , SPEAK_HOURS_ITEM , ci -> talkHours ) ;
			EnableCmd ( EDIT_MENU , SPEAK_HOURS_ITEM ) ;
			CheckCmd ( EDIT_MENU , SPEAK_RANDOM_ITEM , ci -> randomTalk ) ;
			EnableCmd ( EDIT_MENU , SPEAK_RANDOM_ITEM ) ;
			CheckCmd ( EDIT_MENU , SPEAK_OPTION_ITEM , ci -> talkOption ) ;
			EnableCmd ( EDIT_MENU , SPEAK_OPTION_ITEM ) ;
		}
		EnableMenu ( FONT_MENU ) ;
		EnableMenu ( SIZE_MENU ) ;
		EnableMenu ( VOICE_MENU ) ;
		CheckStr ( FONT_MENU , ci -> fontName , 1 ) ;
		CheckStr ( SIZE_MENU , ci -> sizeName , 1 ) ;
		CheckStr ( VOICE_MENU , ci -> voiceName , 1 ) ;
		SizeMenuOutlines ( ci -> fontNum ) ;
	} CLEANUP {
		HSetState ( data , state ) ;
	} DONE ;

	return noErr ;
}


static void
ToggleSecs ( WindowPtr wp , Handle data ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
char state = HGetState ( data ) ;

	HLock ( data ) ;
	TRY {
		ci -> showSecs = ! ci -> showSecs ;
		WindowSize ( ci , wp ) ;
	} CLEANUP {
		HSetState ( data , state ) ;
	} DONE ;
}


static void
ToggleHours ( WindowPtr wp , Handle data ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
char state = HGetState ( data ) ;

	HLock ( data ) ;
	TRY {
		ci -> talkHours = ! ci -> talkHours ;
		if ( ci -> talkHours && ! SpeechAvailable ( ) ) {
			Alert ( 130 , NULL ) ;
		}
	} CLEANUP {
		HSetState ( data , state ) ;
	} DONE ;
}


static void
ToggleOption ( WindowPtr wp , Handle data ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
char state = HGetState ( data ) ;

	HLock ( data ) ;
	TRY {
		ci -> talkOption = ! ci -> talkOption ;
		if ( ci -> talkOption && ! SpeechAvailable ( ) ) {
			Alert ( 130 , NULL ) ;
		}
	} CLEANUP {
		HSetState ( data , state ) ;
	} DONE ;
}


static void
ToggleRandom ( WindowPtr wp , Handle data ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
char state = HGetState ( data ) ;

	HLock ( data ) ;
	TRY {
		ci -> randomTalk = ! ci -> randomTalk ;
		if ( ci -> randomTalk && ! SpeechAvailable ( ) ) {
			Alert ( 130 , NULL ) ;
		}
	} CLEANUP {
		HSetState ( data , state ) ;
	} DONE ;
}


/*	A menu selection was made - take appropriate action			*/
OSErr
ClockCo ( WindowPtr wp , Handle data , short menu , short item ,
	unsigned char * itemStr ) {

ClockInfo * ci = ( ClockInfo * ) * data ;
char state = HGetState ( data ) ;

	TRY {
		if ( menu == FILE_MENU && item == CLOSE_ITEM ) {
			FailErr ( DestroyWindow ( wp ) ) ;
		} else if ( menu == EDIT_MENU ) {
			switch ( item ) {
			case SHOW_SECONDS_ITEM :
				ToggleSecs ( wp , data ) ;
				break ;
			case SPEAK_HOURS_ITEM :
				ToggleHours ( wp , data ) ;
				break ;
			case SPEAK_OPTION_ITEM :
				ToggleOption ( wp , data ) ;
				break ;
			case SPEAK_RANDOM_ITEM :
				ToggleRandom ( wp , data ) ;
				break ;
			default :
				FailErr ( AppCo ( wp , app . data , menu , item , itemStr ) ) ;
				break ;
			}
		} else if ( menu == FONT_MENU ) {
		short sh ;
			GetFNum ( itemStr , & sh ) ;
			if ( itemStr [ 0 ] > 31 ) {
				itemStr [ 0 ] = 31 ;
			}
			CopyPString ( itemStr , ci -> fontName ) ;
			ci -> fontNum = sh ;
			FillFont ( ci , wp ) ;
		} else if ( menu == SIZE_MENU ) {
		long l ;
			StringToNum ( itemStr , & l ) ;
			if ( itemStr [ 0 ] > 15 ) {
				itemStr [ 0 ] = 15 ;
			}
			CopyPString ( itemStr , ci -> sizeName ) ;
			ci -> fontSize = l ;
			FillFont ( ci , wp ) ;
		} else if ( menu == VOICE_MENU ) {
			FailErr ( SetCurrentVoiceName ( itemStr ) ) ;
			CopyPString ( itemStr , ci -> voiceName ) ;
		} else {
			FailErr ( AppCo ( wp , app . data , menu , item , itemStr ) ) ;
		}
	} CLEANUP {
		HSetState ( data , state ) ;
	} DONE ;

	return noErr ;
}


/*	An AppleEvent was received with this window in front		*/
OSErr
ClockAE ( WindowPtr wp , Handle data , AppleEvent * event ,
	AppleEvent * reply ) {
	return noErr ;
}

