/*
 * mytalk.c
 */

#include "speech.h"
#include "mytalk.h"
#include "str.h"


static VoiceSpec defStorage ;
static VoiceSpec * specDefault = NULL ;
static SpeechChannel theChan = NULL ;


static OSErr
AllocChannel ( void ) {

OSErr err = noErr ;

	if ( ! theChan ) {
		err = NewSpeechChannel ( NULL , & theChan ) ;
	}
	if ( ! err ) {
		specDefault = & defStorage ;
		/* err = */ GetSpeechInfo ( theChan , soCurrentVoice , specDefault ) ;
	}
	return err ;
}


OSErr
GetDefaultVoice ( VoiceSpec * spec ) {

OSErr err = noErr ;

	if ( ! specDefault ) {
		err = AllocChannel ( ) ;
	}
	if ( ! err ) {
		* spec = * specDefault ;
	}
	return err ;
}


OSErr
SetDefaultVoice ( VoiceSpec * spec ) {

OSErr err = noErr ;

	if ( ! specDefault ) {
		err = AllocChannel ( ) ;
	}
	if ( ! err ) {
		* specDefault = * spec ;
		err = SetSpeechInfo ( theChan , soCurrentVoice , specDefault ) ;
		if ( err == incompatibleVoice ) {
			err = DisposeSpeechChannel ( theChan ) ;
			theChan = NULL ;
			if ( ! err ) {
				err = NewSpeechChannel ( specDefault , & theChan ) ;
			}
		}
	}

	return err ;
}


OSErr
GetCurrentVoiceName ( unsigned char * name ) {

VoiceSpec spec ;
VoiceDescription info ;
OSErr err = noErr ;

	err = GetDefaultVoice ( & spec ) ;
	if ( ! err ) {
		err = GetVoiceDescription ( & spec , & info , sizeof ( info ) ) ;
	}
	if ( ! err ) {
		CopyPString ( info . name , name ) ;
	}
	return err ;
}


OSErr
SetCurrentVoiceName ( unsigned char * name ) {

VoiceSpec spec ;
VoiceDescription info ;
OSErr err ;
short num , ix ;

	err = CountVoices ( & num ) ;
	for ( ix = 1 ; ix <= num && ! err ; ix ++ ) {
		err = GetIndVoice ( ix , & spec ) ;
		if ( ! err ) {
			err = GetVoiceDescription ( & spec , & info , sizeof ( info ) ) ;
		}
		if ( ! err ) {
			if ( EqualString ( name , info . name , 0 , 1 ) ) {
				err = SetDefaultVoice ( & spec ) ;
				return err ;
			}
		}
	}
	if ( ! err ) {
		return GetCurrentVoiceName ( name ) ;
	}
	return err ;
}


OSErr
MySpeakString ( unsigned char * string ) {

short err = noErr ;

	if ( ! theChan ) {
		err = AllocChannel ( ) ;
	}
	if ( ! err ) {
		err = SetSpeechInfo ( theChan , soCurrentA5 ,
			( void * ) ( * ( long * ) 0x904 ) ) ;
	}
	if ( ! err ) {
		err = SpeakText ( theChan , ( Ptr ) string + 1 , * string ) ;
	}
	return err ;
}


OSErr
ShutdownMyTalk ( long waitDone ) {

OSErr err = noErr ;

	if ( waitDone ) {
	long timeOut = TickCount ( ) + waitDone ;
		while ( SpeechBusy ( ) ) {
			SystemTask ( ) ;
			if ( TickCount ( ) > timeOut ) {
				break ;
			}
		}
	}
	specDefault = NULL ;
	if ( theChan ) {
		err = DisposeSpeechChannel ( theChan ) ;
		theChan = NULL ;
	}
	return err ;
}
